// $Id: Application.cpp,v 1.30 2012/09/05 22:08:37 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file Application.cpp
\brief application Main Window singleton object
\author  Hugo Pereira
\version $Revision: 1.30 $
\date $Date: 2012/09/05 22:08:37 $
*/

#include "Application.h"

#include "Config.h"
#include "ConfigurationDialog.h"
#include "CvsCommandDialog.h"
#include "Icons.h"
#include "IconEngine.h"
#include "LogDialog.h"
#include "MainWindow.h"
#include "Menu.h"
#include "QuestionDialog.h"
#include "QtUtil.h"
#include "ScratchFileMonitor.h"
#include "Util.h"
#include "XmlFileList.h"
#include "XmlOptions.h"

#include <QtGui/QApplication>

//____________________________________________
void Application::usage( void )
{

    Debug::Throw(0) << "Usage : cvswatch [options] [file]" << endl;
    SERVER::ApplicationManager::commandLineParser().usage();
    return;

}

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( 0, arguments ),
    Counter( "Application" ),
    busyCount_( 0 ),
    recentFiles_( 0 ),
    mainWindow_( 0 )
{}

//____________________________________________
Application::~Application( void )
{}

//____________________________________________
void Application::initApplicationManager( void )
{
    Debug::Throw( "Application::initApplicationManager.\n" );

    // retrieve files from arguments and expand if needed
    CommandLineParser parser( SERVER::ApplicationManager::commandLineParser( _arguments() ) );
    QStringList& orphans( parser.orphans() );
    for( QStringList::iterator iter = orphans.begin(); iter != orphans.end(); ++iter )
    { if( !iter->isEmpty() ) (*iter) = File( *iter ).expand(); }

    // replace arguments
    _setArguments( parser.arguments() );

    // base class initialization
    BaseApplication::initApplicationManager();

}

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // rename about action
    aboutAction().setText( "About Cvswatch" );
    configurationAction().setText( "Configure Cvswatch..." );

    // scratch files
    scratchFileMonitor_ = new ScratchFileMonitor( this );
    connect( qApp, SIGNAL( aboutToQuit( void ) ), scratchFileMonitor_, SLOT( deleteScratchFiles( void ) ) );

    // recent files
    recentFiles_ = new XmlFileList();
    recentFiles_->setCheck( true );

    // insert log list
    mainWindow_ = new MainWindow();

    mainWindow().centerOnDesktop();
    mainWindow().setWindowTitle( "Cvswatch" );
    mainWindow().show();

    QStringList filenames( SERVER::ApplicationManager::commandLineParser( _arguments() ).orphans() );
    if( !filenames.isEmpty() ) mainWindow().open( FileRecord( File( filenames.front() ).expand() ) );
    else mainWindow().open( FileRecord( Util::workingDirectory() ) );

    emit configurationChanged();

    return true;

}

//___________________________________________________________
void Application::busy( void )
{
    if( !busyCount_ ) BaseApplication::busy();
    busyCount_ ++;
}

//____________________________________________________________
void Application::idle( void )
{
    if( busyCount_ > 0 ) busyCount_ --;
    if( !busyCount_ ) BaseApplication::idle();
}

//___________________________________________________________
void Application::_configuration( void )
{
    Debug::Throw( "Application::configuration.\n" );
    emit saveConfiguration();
    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL( configurationChanged( void ) ), SIGNAL( configurationChanged( void ) ) );
    connect( &dialog, SIGNAL( clearCache( void ) ), &mainWindow(), SLOT( clearCache( void ) ) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();
}

//________________________________________________
bool Application::_processCommand( SERVER::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == SERVER::ServerCommand::RAISE )
    {
        if( mainWindow_ ) mainWindow().uniconify();

        QStringList filenames( SERVER::ApplicationManager::commandLineParser( command.arguments() ).orphans() );
        if( !filenames.isEmpty() )
        {

            QString buffer;
            QTextStream( &buffer ) << "Accept request for file \"" << filenames.front() << "\" ?";
            if( QuestionDialog( mainWindow_, buffer ).centerOnParent().exec() )
            { mainWindow().open( File( filenames.front() ) ); }
        }

        return true;
    } else return false;

}
