// $Id: ConfigurationDialog.cpp,v 1.48 2012/09/05 18:48:08 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/


#include "AnimatedTreeView.h"
#include "Application.h"
#include "BaseIcons.h"
#include "ConfigurationDialog.h"
#include "CustomToolBar.h"
#include "Debug.h"
#include "FileList.h"
#include "FileSelectionFrame.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "MainWindow.h"
#include "OptionSpinBox.h"
#include "OptionBrowsedLineEditor.h"
#include "OptionCheckBox.h"
#include "QuestionDialog.h"
#include "RecentFilesConfiguration.h"
#include "Singleton.h"
#include "TreeViewConfiguration.h"

#include <QtGui/QGroupBox>
#include <QtGui/QLabel>
#include <QtGui/QPushButton>

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent ):
    BaseConfigurationDialog( parent )
{


    Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );
    setWindowTitle( "Configuration - Cvswatch" );

    // base configuration
    baseConfiguration();

    // main window
    QWidget* page;
    OptionCheckBox* checkbox;

    page = &addPage( IconEngine::get( ICONS::PREFERENCE_APPEARANCE), "Appearance", "Main window appearance" );
    QWidget *box = new QWidget( page );
    page->layout()->addWidget( box );

    QVBoxLayout* layout = new QVBoxLayout();
    layout->setMargin(0);
    layout->setSpacing(5);
    box->setLayout( layout );

    GridLayout* gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    layout->addLayout( gridLayout );

    // icon view pixmap size
    QLabel* label;
    gridLayout->addWidget( label = new QLabel( "Icon size in icon view:", box ) );
    OptionSpinBox *spinbox = new OptionSpinBox( box, "ICON_VIEW_PIXMAP_SIZE" );
    spinbox->setMinimum( 8 );
    spinbox->setMaximum( 128 );
    spinbox->setSuffix( "px" );
    gridLayout->addWidget( spinbox );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    // full path
    layout->addWidget( checkbox = new OptionCheckBox( "Show full path inside location bar", box, "USE_TRUNCATION" ) );
    checkbox->setNegative( true );
    addOptionWidget( checkbox );

    // navigation
    box->layout()->addWidget( checkbox = new OptionCheckBox( "Show parent directory icon in file list", box, "SHOW_NAVIGATOR" ) );
    addOptionWidget( checkbox );

    // tooltips
    layout->addWidget( checkbox = new OptionCheckBox( "Show tooltips in icon view", box, "SHOW_TOOLTIPS" ) );
    addOptionWidget( checkbox );

    // hidden files
    layout->addWidget( checkbox = new OptionCheckBox( "Show hidden files ", box, "SHOW_HIDDEN_FILES" ) );
    addOptionWidget( checkbox );

    // recent files
    page = &addPage( IconEngine::get( ICONS::PREFERENCE_RECENT_FILES ), "Recent files", "Recent files list settings", true );
    RecentFilesConfiguration* recent_files_configuration = new RecentFilesConfiguration( page, Singleton::get().application<Application>()->recentFiles() );
    page->layout()->addWidget( recent_files_configuration );
    addOptionWidget( recent_files_configuration );

    // Cache
    page = &addPage( IconEngine::get( ICONS::PREFERENCE_CACHE ), "Cache", "Configure the cache used to store CVS information" );

    //
    box = new QWidget( page );
    page->layout()->addWidget( box );

    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setSpacing(5);
    hLayout->setMargin(5);
    box->setLayout( hLayout );

    layout = new QVBoxLayout();
    layout->setMargin(0);
    layout->setSpacing(5);
    hLayout->addLayout( layout );
    hLayout->addStretch();

    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(0);
    gridLayout->setMaxCount(2);
    layout->addLayout( gridLayout );

    gridLayout->addWidget( checkbox = new OptionCheckBox( "Limit revision cache size to ", box, "LIMIT_REVISION_CACHE" ) );
    checkbox->setToolTip( "Enable/disable the limitation of the file revision cache to a maximum size" );
    addOptionWidget( checkbox );

    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "REVISION_CACHE_SIZE" ) );
    spinbox->setMinimum( 0 );
    spinbox->setMaximum( 1000 );
    spinbox->setSuffix( "MiB" );
    spinbox->setToolTip( "Maximum size of the revision cache" );
    addOptionWidget( spinbox );

    gridLayout->addWidget( checkbox = new OptionCheckBox( "Limit file information cache size to ", box, "LIMIT_FILEINFO_CACHE" ) );
    checkbox->setToolTip( "Enable/disable the limitation of the file information cache to a maximum size" );
    addOptionWidget( checkbox );

    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "FILEINFO_CACHE_SIZE" ) );
    spinbox->setMinimum( 0 );
    spinbox->setMaximum( 1000 );
    spinbox->setSuffix( "MiB" );
    spinbox->setToolTip( "Maximum size of the revision cache" );
    addOptionWidget( spinbox );

    hLayout = new QHBoxLayout();
    hLayout->setSpacing(5);
    hLayout->setMargin(0);
    layout->addLayout( hLayout );

    hLayout->addStretch();
    hLayout->addWidget( label = new QLabel( "<a href='clear'><u>Clear recent file information and revisions cache...</u></a>", box ) );
    connect( label, SIGNAL( linkActivated( const QString& ) ), SLOT( _clearCache( void ) ) );

    // commands
    page = &addPage( IconEngine::get( ICONS::PREFERENCE_FILE_TYPES ), "Applications", "Third-party applications" );
    box = new QGroupBox( page );
    page->layout()->addWidget( box );

    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight | Qt::AlignVCenter );
    box->setLayout( gridLayout );

    OptionBrowsedLineEditor* editor;
    gridLayout->addWidget( label = new QLabel( "Cvs:", box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "CVS" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    gridLayout->addWidget( label = new QLabel( "Text editor:", box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "EDITOR" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    gridLayout->addWidget( label = new QLabel( "Diff text editor:", box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "DIFF_EDITOR" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    box = new QGroupBox( "Environment", page );
    page->layout()->addWidget( box );

    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight | Qt::AlignVCenter );
    box->setLayout( gridLayout );

    gridLayout->addWidget( label = new QLabel( "CVS root:", box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "DEFAULT_CVSROOT" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    // load initial configuration
    read();

}

//_________________________________________________________
void ConfigurationDialog::_clearCache( void )
{
    if( !QuestionDialog( this, "Permanently clear revisions and file informations caches ?" ).exec() ) return;
    emit clearCache();
}
