// $Id: CvsCommandModel.cpp,v 1.20 2012/02/20 21:04:51 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "CustomPixmap.h"
#include "CvsCommandModel.h"
#include "Icons.h"
#include "Singleton.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QtGui/QIcon>
#include <cassert>

//_______________________________________________
const QString CvsCommandModel::columnTitles_[ CvsCommandModel::nColumns ] =
{
    "Command",
    "Path"
};

//_______________________________________________
CvsCommandModel::IconCache& CvsCommandModel::_icons( void )
{
    static IconCache singleton;
    return singleton;
}

//_______________________________________________________________
CvsCommandModel::CvsCommandModel( QObject* parent ):
    ListModel<CvsCommand>( parent ),
    Counter( "CvsCommandModel" )
{

    Debug::Throw( "CvsCommandModel::CvsCommandModel.\n" );
    connect( Singleton::get().application(), SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    _updateConfiguration();

}

//__________________________________________________________________
QVariant CvsCommandModel::data( const QModelIndex& index, int role ) const
{
    Debug::Throw( "CvsCommandModel::data.\n" );

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated command
    const CvsCommand& command( get()[index.row()] );

    // return text associated to command and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {
            case NAME:
            return QString( command.name() );

            case PATH:
            return command.argumentCount() > 0  ? command.argument(0):QVariant();

            default:
            return QVariant();
        }
    }

    // return icon associated to file
    if( role == Qt::DecorationRole && index.column() == 0 )
    { return _icons()[command.type()]; }

    return QVariant();

}

//__________________________________________________________________
QVariant CvsCommandModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//____________________________________________________________
void CvsCommandModel::add( CvsCommand command )
{
    Debug::Throw( "CvsCommandModel::add.\n" );
    emit layoutAboutToBeChanged();
    _get().push_back( command );
    emit layoutChanged();
}

//____________________________________________________________
void CvsCommandModel::remove( CvsCommand command )
{

    Debug::Throw( "CvsCommandModel::remove.\n" );

    List::iterator iter = std::find_if( _get().begin(), _get().end(), CvsCommand::SameTypeFTor( command.type() ) );
    if( iter != _get().end() )
    {
        emit layoutAboutToBeChanged();
        _get().erase( iter );
        emit layoutChanged();
    }

    return;

}

//_________________________________________________________
void CvsCommandModel::_updateConfiguration( void )
{

    Debug::Throw( "CvsCommandModel::_updateConfiguration.\n" );

    // install pixmaps
    _icons().clear();
    _installIcons();

    reset();

}

//_____________________________________________________________________
void CvsCommandModel::_installIcons( void ) const
{

    Debug::Throw( "CvsCommandList::_installIcons" );

    if( !_icons().empty() ) return;

    // load pixmaps
    const unsigned int pixmapSize = XmlOptions::get().get<unsigned int>( "LIST_ICON_SIZE" );

    QSize size( pixmapSize, pixmapSize );
    QSize scale = 0.75*size;

    // type icons
    typedef QHash< CvsCommand::Type, QString > IconNames;
    IconNames typeNames;
    typeNames[CvsCommand::STATUS] = ICONS::REMOTELY_MODIFIED;
    typeNames[CvsCommand::LOG] = ICONS::REMOTELY_MODIFIED;
    typeNames[CvsCommand::FAKE_UPDATE] = ICONS::REMOTELY_MODIFIED;
    typeNames[CvsCommand::CHECKOUT] = ICONS::REMOTELY_ADDED;
    typeNames[CvsCommand::UPDATE] = ICONS::REMOTELY_MODIFIED;
    typeNames[CvsCommand::ADD] = ICONS::LOCALLY_ADDED;
    typeNames[CvsCommand::REMOVE] = ICONS::LOCALLY_REMOVED;
    typeNames[CvsCommand::COMMIT] = ICONS::LOCALLY_MODIFIED;
    typeNames[CvsCommand::CHECKOUT_REVISION] = ICONS::REMOTELY_ADDED;

    // load link pixmap
    for( IconNames::iterator iter = typeNames.begin(); iter != typeNames.end(); ++iter )
    {

        _icons()[iter.key()] = CustomPixmap().empty( size )
            .merge( CustomPixmap().find( iter.value() )
            .scaled( scale, Qt::KeepAspectRatio, Qt::SmoothTransformation ),
            CustomPixmap::CENTER );

    }

    return;
}
