// $Id: FileInfoModel.cpp,v 1.31 2012/09/05 10:14:37 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "CustomPixmap.h"
#include "FileInfoModel.h"
#include "Icons.h"
#include "Singleton.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QtGui/QIcon>

//_______________________________________________________________
FileInfoModel::FileInfoModel( QObject* parent ):
    BaseFileInfoModel<FileInfo>( parent ),
    Counter( "FileInfoModel" )
{
    Debug::Throw( "FileInfoModel::FileInfoModel.\n" );

    connect( Singleton::get().application(), SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    _updateConfiguration();

}

//__________________________________________________________________
QVariant FileInfoModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // return icon associated to file
    if( showIcons() && role == Qt::DecorationRole && index.column() == 0 )
    {
        const FileInfo& info( get()[index.row()] );
        unsigned int type( info.type()&(FileInfo::DOCUMENT|FileInfo::FOLDER|FileInfo::NAVIGATOR|FileInfo::LINK) );
        type|= (info.status()<<FileInfo::SHIFT);
        return _constIcons()[type];
    }

    // fallback to parent
    return BaseFileInfoModel<FileInfo>::data( index, role );

}

//_________________________________________________________
void FileInfoModel::_updateConfiguration( void )
{

    Debug::Throw( "FileInfoModel::_updateConfiguration.\n" );

    // install pixmaps
    _icons().clear();
    _installIcons();
    reset();

}

//_____________________________________________________________________
void FileInfoModel::_installIcons( void )
{

    Debug::Throw( "FileInfoModel::_installIcons.\n" );

    if( !_icons().empty() ) return;

    // pixmap size
    unsigned int pixmap_size = XmlOptions::get().get<unsigned int>( "LIST_ICON_SIZE" );
    QSize size( pixmap_size, pixmap_size );
    QSize scale(size*0.9);

    // type icons
    typedef QHash< int, QString > IconNames;

    // small icons
    IconNames typeNames;
    typeNames[FileInfo::DOCUMENT] = ICONS::DOCUMENT;
    typeNames[FileInfo::FOLDER] = ICONS::FOLDER;
    typeNames[FileInfo::NAVIGATOR] = ICONS::PARENT_DIRECTORY;
    CustomPixmap link = CustomPixmap().find( ICONS::LINK );
    for( IconNames::iterator iter = typeNames.begin(); iter != typeNames.end(); ++iter )
    {

        unsigned int type( iter.key()|(FileInfo::UNKNOWN<<FileInfo::SHIFT) );
        _icons()[type] = CustomPixmap()
            .empty( size )
            .merge( CustomPixmap().find( iter.value() )
            .scaled( scale, Qt::KeepAspectRatio, Qt::SmoothTransformation ), CustomPixmap::CENTER );

        _icons()[type | FileInfo::LINK] = CustomPixmap()
            .empty( size )
            .merge( CustomPixmap().find( iter.value() )
            .merge( link, CustomPixmap::BOTTOM_LEFT )
            .scaled( scale, Qt::KeepAspectRatio, Qt::SmoothTransformation ), CustomPixmap::CENTER );

    }

    // large icons
    IconNames typeNamesLarge;
    typeNamesLarge[FileInfo::DOCUMENT] = ICONS::DOCUMENT_LARGE;
    typeNamesLarge[FileInfo::FOLDER] = ICONS::FOLDER_LARGE;
    CustomPixmap linkOverlayLarge = CustomPixmap().find( ICONS::LINK_LARGE );
    for( IconNames::iterator iter = typeNamesLarge.begin(); iter != typeNamesLarge.end(); ++iter )
    {
        unsigned int type( iter.key()|(FileInfo::UNKNOWN<<FileInfo::SHIFT) );
        _icons()[type].addPixmap( CustomPixmap().find( iter.value() ) );
        _icons()[type|FileInfo::LINK].addPixmap( CustomPixmap().find( iter.value() ).merge( linkOverlayLarge, CustomPixmap::BOTTOM_RIGHT ) );
    }

    // status icons
    IconNames statusNames;
    statusNames[FileInfo::UNKNOWN] = ICONS::UNKNOWN;
    statusNames[FileInfo::UP_TO_DATE] = ICONS::UP_TO_DATE;
    statusNames[FileInfo::LOCALLY_MODIFIED] = ICONS::LOCALLY_MODIFIED;
    statusNames[FileInfo::LOCALLY_ADDED] = ICONS::LOCALLY_ADDED;
    statusNames[FileInfo::LOCALLY_REMOVED] = ICONS::LOCALLY_REMOVED;
    statusNames[FileInfo::REMOTELY_MODIFIED] = ICONS::REMOTELY_MODIFIED;
    statusNames[FileInfo::REMOTELY_ADDED] = ICONS::REMOTELY_ADDED;
    statusNames[FileInfo::REMOTELY_REMOVED] = ICONS::REMOTELY_REMOVED;
    statusNames[FileInfo::MERGE] = ICONS::MERGE;
    statusNames[FileInfo::CONFLICT] = ICONS::CONFLICT;

    CustomPixmap base(
        CustomPixmap().find( typeNames[FileInfo::DOCUMENT] )
        .scaled( size, Qt::KeepAspectRatio, Qt::SmoothTransformation ) );

    CustomPixmap baseLarge( CustomPixmap().find( typeNamesLarge[FileInfo::DOCUMENT] ) );
    for( IconNames::iterator iter = statusNames.begin(); iter != statusNames.end(); ++iter )
    {
        const int type( FileInfo::DOCUMENT | (iter.key() << FileInfo::SHIFT ) );
        _icons()[ type ] = CustomPixmap()
            .empty( size )
            .merge( base
            .merge(CustomPixmap()
            .find( iter.value() )
            .scaled( 0.6*size, Qt::KeepAspectRatio, Qt::SmoothTransformation ) )
            .scaled( scale, Qt::KeepAspectRatio, Qt::SmoothTransformation ), CustomPixmap::CENTER );

        // large  icon
        _icons()[ type ].addPixmap(
            baseLarge.merge( CustomPixmap()
            .find( iter.value() )
            .scaled( 0.4*baseLarge.size(), Qt::KeepAspectRatio, Qt::SmoothTransformation ) ) );


    }

    return;
}
