// $Id: MainWindow.cpp,v 1.54 2012/09/05 22:08:37 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "MainWindow.h"
#include "AnimatedTreeView.h"
#include "Application.h"
#include "ClockLabel.h"
#include "Command.h"
#include "AnimatedTextEditor.h"
#include "FileDialog.h"
#include "CustomToolBar.h"
#include "CvsCommandDialog.h"
#include "FileInfoCacheDialog.h"
#include "FileList.h"
#include "FileSelectionFrame.h"
#include "Icons.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "LogDialog.h"
#include "LogWidget.h"
#include "Menu.h"
#include "PathEditor.h"
#include "QtUtil.h"
#include "RecentFilesMenu.h"
#include "RevisionCacheDialog.h"
#include "RevisionFrame.h"
#include "ScratchFileMonitor.h"
#include "Singleton.h"
#include "StatusBar.h"
#include "Util.h"
#include "ViewModeMenu.h"
#include "XmlOptions.h"

#include <QtCore/QDir>
#include <QtGui/QApplication>
#include <QtGui/QLabel>
#include <QtGui/QGroupBox>
#include <QtGui/QSplitter>

//___________________________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    Counter( "MainWindow" )
{

    setOptionName( "MAIN_WINDOW" );

    Debug::Throw( "MainWindow::MainWindow.\n" );
    _installActions();
    Application& application( *Singleton::get().application<Application>() );
    addAction( &application.closeAction() );

    // main widget and layout
    QWidget* main = new QWidget( this );
    QVBoxLayout* mainLayout( new QVBoxLayout() );
    main->setLayout( mainLayout );
    mainLayout->setMargin(0);
    mainLayout->setSpacing(0);
    setCentralWidget( main );

    // splitter
    QSplitter *splitter = new QSplitter( main );
    mainLayout->addWidget( splitter, 1 );
    splitter->setOrientation( Qt::Horizontal );

    //! dialogs
    commandDialog_ = new CvsCommandDialog();
    logDialog_ = new LogDialog();

    connect( commandDialog_, SIGNAL( dialogHidden() ), SLOT( _commandDialogHidden() ) );
    connect( logDialog_, SIGNAL( dialogHidden() ), SLOT( _logDialogHidden() ) );

    // file selection frame on the left
    fileSelectionFrame_ = new FileSelectionFrame( splitter );
    fileSelectionFrame_->editor().setParent( main );
    mainLayout->insertWidget( 0, &fileSelectionFrame_->editor() );

    // second splitter on the right
    splitter = new QSplitter( splitter );
    splitter->setOrientation( Qt::Vertical );
    revisionFrame_ = new RevisionFrame( splitter );

    if( XmlOptions::get().contains( "REVISION_FRAME_HEIGHT" ) )
    { revisionFrame_->setDefaultHeight( XmlOptions::get().get<int>( "REVISION_FRAME_HEIGHT" ) ); }

    // right vbox for buttons and comments
    QWidget *right = new QWidget( splitter );
    QVBoxLayout* vLayout = new QVBoxLayout();
    right->setLayout( vLayout );
    vLayout->setSpacing(0);
    vLayout->setMargin(0);

    // comments
    QLabel* label;
    vLayout->addWidget( label = new QLabel( "Comments:", right ) );
    vLayout->addWidget( commentsEditor_ = new AnimatedTextEditor( right ), 1 );

    label->setMargin( 5 );
    label->setBuddy( commentsEditor_ );

    commentsEditor_->setReadOnly( true );
    commentsEditor_->setWrapFromOptions( false );
    commentsEditor_->wrapModeAction().setChecked( true );
    commentsEditor_->setFrameStyle( QFrame::NoFrame );

    commentsEditor_->setForegroundRole( QPalette::WindowText );
    commentsEditor_->setBackgroundRole( QPalette::Window );

    commentsEditor_->viewport()->setForegroundRole( QPalette::WindowText );
    commentsEditor_->viewport()->setBackgroundRole( QPalette::Window );
    commentsEditor_->blockHighlight().setEnabled( false );

    // menu bar
    setMenuBar( menu_ = new Menu( this ) );
    connect( &menu_->viewModeMenu(), SIGNAL( viewModeSelected( FileSelectionFrame::ViewMode ) ), fileSelectionFrame_, SLOT( setViewMode( FileSelectionFrame::ViewMode ) ) );

    // status bar
    setStatusBar( statusbar_ = new StatusBar( this ) );
    statusbar_->addLabels(1, 1 );
    statusbar_->addClock();

    // navigation toolbar
    CustomToolBar* toolbar;
    toolbar = new CustomToolBar( "Navigation", this, "NAVIGATION_TOOLBAR" );
    toolbar->addAction( &selectionFrame().previousDirectoryAction() );
    toolbar->addAction( &selectionFrame().nextDirectoryAction() );
    toolbar->addAction( &selectionFrame().parentDirectoryAction() );
    toolbar->addAction( &selectionFrame().homeDirectoryAction() );
    toolbar->addAction( &reloadAction() );

    toolbar->addSeparator();
    toolbar->addAction( &selectionFrame().cvsUpdateDirectoryAction() );
    toolbar->addAction( &selectionFrame().cvsCheckOutAction() );

    toolbar->addSeparator();
    toolbar->addAction( &selectionFrame().cvsAddAction() );
    toolbar->addAction( &selectionFrame().cvsRemoveAction() );
    toolbar->addAction( &selectionFrame().cvsCommitDirectoryAction() );

    // connections
    // file editor
    connect( &fileSelectionFrame_->editor(), SIGNAL( pathChanged( const File& ) ), SLOT( _reload( void ) ) );
    connect( &commandDialog(), SIGNAL( commandAborted( CvsCommand ) ), &selectionFrame().interface(), SLOT( abort( CvsCommand ) ) );
    connect( &commandDialog(), SIGNAL( commandAborted( CvsCommand ) ), &revisionFrame().interface(), SLOT( abort( CvsCommand ) ) );

    connect( &selectionFrame(), SIGNAL( fileSelected( File ) ), SLOT( _setFile( File ) ) );
    connect( &selectionFrame().interface(), SIGNAL( commandStarted( CvsCommand ) ), Singleton::get().application(), SLOT( busy() ) );
    connect( &selectionFrame().interface(), SIGNAL( commandCompleted( CvsCommand ) ), Singleton::get().application(), SLOT( idle() ) );
    connect( &selectionFrame().interface(), SIGNAL( commandAdded( CvsCommand ) ), &commandDialog().model(), SLOT( add( CvsCommand ) ) );
    connect( &selectionFrame().interface(), SIGNAL( commandRemoved( CvsCommand ) ), &commandDialog().model(), SLOT( remove( CvsCommand ) ) );
    connect( &selectionFrame().interface(), SIGNAL( commandCompleted( CvsCommand ) ), &commandDialog().model(), SLOT( remove( CvsCommand ) ) );
    connect( &selectionFrame().interface(), SIGNAL( commandCompleted( CvsCommand ) ), &revisionFrame(), SLOT( commandCompleted( CvsCommand ) ) );
    connect( &selectionFrame().interface(), SIGNAL( messageAvailable( QString, unsigned int, QColor, int ) ), &logDialog().editor(), SLOT( append( QString, unsigned int, QColor, int ) ) );
    connect( &selectionFrame(), SIGNAL( messageAvailable( QString, unsigned int, QColor, int ) ), &logDialog().editor(), SLOT( append( QString, unsigned int, QColor, int ) ) );

    connect( &selectionFrame().interface(), SIGNAL( commandStarted( CvsCommand ) ), SLOT( _updateStatusBar( CvsCommand ) ) );
    connect( &selectionFrame().interface(), SIGNAL( commandCompleted( CvsCommand ) ), &statusbar_->label(), SLOT( clear() ) );

    connect( &revisionFrame(), SIGNAL( currentRevisionChanged( Revision ) ), SLOT( _displayComments( Revision ) ) );
    connect( &revisionFrame().interface(), SIGNAL( messageAvailable( QString, unsigned int, QColor, int ) ), &logDialog().editor(), SLOT( append( QString, unsigned int, QColor, int ) ) );
    connect( &revisionFrame().interface(), SIGNAL( commandStarted( CvsCommand ) ), Singleton::get().application(), SLOT( busy() ) );
    connect( &revisionFrame().interface(), SIGNAL( commandCompleted( CvsCommand ) ), Singleton::get().application(), SLOT( idle() ) );
    connect( &revisionFrame().interface(), SIGNAL( commandAdded( CvsCommand ) ), &commandDialog().model(), SLOT( add( CvsCommand ) ) );
    connect( &revisionFrame().interface(), SIGNAL( commandRemoved( CvsCommand ) ), &commandDialog().model(), SLOT( remove( CvsCommand ) ) );
    connect( &revisionFrame().interface(), SIGNAL( commandCompleted( CvsCommand ) ), &commandDialog().model(), SLOT( remove( CvsCommand ) ) );

    // propagate command completed from revision frame to FileSelectionFrame
    // this to allow diff actions and checkout revision actions to toggle update of the list
    connect( &revisionFrame().interface(), SIGNAL( commandCompleted( CvsCommand ) ), &selectionFrame(), SLOT( commandCompleted( CvsCommand ) ) );
    connect( &revisionFrame().interface(), SIGNAL( commandStarted( CvsCommand ) ), SLOT( _updateStatusBar( CvsCommand ) ) );
    connect( &revisionFrame().interface(), SIGNAL( commandCompleted( CvsCommand ) ), &statusbar_->label(), SLOT( clear() ) );
    connect( &revisionFrame().interface(), SIGNAL( scratchFileAdded( const File& ) ), &Singleton::get().application<Application>()->scratchFileMonitor(), SLOT( add( const File& ) ) );

    connect( &application, SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    connect( qApp, SIGNAL( aboutToQuit() ), SLOT( _saveConfiguration() ) );
    _updateConfiguration();

    // read cache and assign to children
    cache_.read( QString( XmlOptions::get().raw( "CACHE_FILE" ) ) );
    selectionFrame().setCache( cache_.fileInfoCache() );
    revisionFrame().setCache( cache_.revisionCache() );

    // splitter
    splitter->setStretchFactor( 0, 0 );
    splitter->setStretchFactor( 1, 1 );
    connect( splitter, SIGNAL( splitterMoved( int, int ) ), SLOT( _splitterMoved( void ) ) );

}

//__________________________________________________________________________
void MainWindow::open( FileRecord record )
{

    Debug::Throw( "MainWindow::open.\n" );

    if( record.file().isEmpty() )
    {
        QString file( FileDialog( this ).selectFile( selectionFrame().path() ).getFile() );
        if( file.isNull() ) return;
        else record = FileRecord( file );
    }

    // update file selection frame
    selectionFrame().setFile( record.file() );

    // update fileEditor and menu
    _setFile( record.file() );

}

//_______________________________________________________
void MainWindow::clearCache( void )
{
    cache_.clear();
    fileSelectionFrame_->clearCache();
    revisionFrame_->clearCache();
}

//_______________________________________________________
void MainWindow::timerEvent( QTimerEvent* event )
{

    if( event->timerId() == resizeTimer_.timerId() )
    {

        // stop timer
        resizeTimer_.stop();

        // save size
        XmlOptions::get().set<int>( "REVISION_FRAME_HEIGHT", revisionFrame().height() );

    } else return BaseMainWindow::timerEvent( event );

}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    _saveConfiguration();
    event->accept();
    qApp->quit();
}

//_______________________________________
void MainWindow::_updateConfiguration( void )
{

    Debug::Throw( "MainWindow::_updateConfiguration" );

    // command dialog
    if( XmlOptions::get().get<bool>( "SHOW_COMMAND_LIST" ) ) _showCommandDialog();
    else commandDialog().hide();

    // log dialog
    if( XmlOptions::get().get<bool>( "SHOW_LOG" ) ) _showLogDialog();
    else logDialog().hide();

    // reset comments editor
    commentsEditor_->blockHighlight().setEnabled( false );

}

///_______________________________________
void MainWindow::_saveConfiguration( void )
{

    Debug::Throw( "MainWindow::_saveConfiguration.\n" );

    // retrieve cache from children and write to file
    const qreal fileInfoScale( 1024.0*1024.0/182 );
    const qreal revisionScale( 1024.0*1024.0/153 );

    const int maxFileInfoCacheSize( fileInfoScale*XmlOptions::get().get<int>("FILEINFO_CACHE_SIZE") );
    const int maxRevisionCacheSize( revisionScale*XmlOptions::get().get<int>("REVISION_CACHE_SIZE") );
    cache_.setFileInfoCache( selectionFrame().cache().setMaximumSize( XmlOptions::get().get<bool>( "LIMIT_FILEINFO_CACHE" ) ? maxFileInfoCacheSize : -1 ).truncate() );
    cache_.setRevisionCache( revisionFrame().cache().setMaximumSize( XmlOptions::get().get<bool>( "LIMIT_REVISION_CACHE" ) ? maxRevisionCacheSize:-1 ).truncate() );
    cache_.write();

}

//__________________________________________________________________________
void MainWindow::_reload( void )
{

    Debug::Throw( "MainWindow::_reload.\n" );

    // reload information based on editor contents
    QFileInfo file( fileSelectionFrame_->editor().path() );
    QFileInfo path( file.absolutePath() );
    if( path.exists() )
    {

        Debug::Throw() << "MainWindow::_reload - file: " << path.absoluteFilePath() << endl;
        selectionFrame().setFile( file.absoluteFilePath() );

    } else {

        // invalid path. Prompt dialog
        QString buffer;
        QTextStream( &buffer ) << "Path " << path.absoluteFilePath() << " is invalid. <Reload> canceled.";
        InformationDialog( this, buffer ).exec();

    }

}

//__________________________________________________________________________
void MainWindow::_setFile( File file )
{

    Debug::Throw( "MainWindow::_setFile.\n" );

    // convert file to canonical before further updates.
    file = QFileInfo( file ).canonicalFilePath();

    // clear comments if file differ from current revision file
    if( file != revisionFrame().file() )
    {
        revisionFrame().setFile( file.isDirectory() ? File():file );
        commentsEditor_->QTextEdit::clear();
    }

    // add to openPrevious menu
    Singleton::get().application<Application>()->recentFiles().add( file.isDirectory() ? file:file.path() );
    menu().recentFilesMenu().setCurrentFile( file.isDirectory() ? file:file.path() );

    return;
}

//_______________________________________________________________________
void MainWindow::_updateStatusBar( CvsCommand command )
{
    Debug::Throw( "MainWindow::_updateStatusBar.\n" );

    // create stream, add name, and possible argument
    QString buffer;
    QTextStream what( &buffer );
    what << command.name();

    if( command.argumentCount() )
    { what << " " << command.argument(0); }

    // update status bar
    statusbar_->label(0).setText( buffer );

}
//__________________________________________________________________________
void MainWindow::_showCommandDialog( void )
{

    Debug::Throw( "MainWindow::_showCommandDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", true );
    commandDialog().show();
    commandDialog().raise();

}

//_______________________________________________
void MainWindow::_commandDialogHidden( void )
{
    Debug::Throw( "Application::_commandDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", false );
}

//__________________________________________________________________________
void MainWindow::_showLogDialog( void )
{

    Debug::Throw( "MainWindow::_showLogDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", true );
    logDialog().show();
    logDialog().raise();

}

//_______________________________________________
void MainWindow::_logDialogHidden( void )
{
    Debug::Throw( "MainWindow::_logDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", false );
}

//_______________________________________________________________________
void MainWindow::_showFileInfoCache( void )
{

    Debug::Throw( "MainWindow::_showFileInfoCache.\n" );
    FileInfoCacheDialog dialog( this );
    dialog.setCache( cache_.fileInfoCache() );
    dialog.centerOnParent().exec();

}

//_______________________________________________________________________
void MainWindow::_showRevisionCache( void )
{

    Debug::Throw( "MainWindow::_showRevisionCache.\n" );
    RevisionCacheDialog dialog( this );
    dialog.setCache( cache_.revisionCache() );
    dialog.centerOnParent().exec();

}

//________________________________________________________
void MainWindow::_splitterMoved( void )
{
    Debug::Throw( "MainWindow::_splitterMoved.\n" );
    resizeTimer_.start( 200, this );
}

//_______________________________________________________________________
void MainWindow::_installActions( void )
{
    Debug::Throw( "MainWindow::_installActions.\n" );

    addAction( reloadAction_ = new QAction( IconEngine::get( ICONS::RELOAD ), "Reload", this ) );
    reloadAction_->setShortcut( QKeySequence::Refresh );
    connect( reloadAction_, SIGNAL( triggered() ), SLOT( _reload() ) );

    addAction( openAction_ = new QAction( IconEngine::get( ICONS::OPEN ), "Open...", this ) );
    openAction_->setShortcut( QKeySequence::Open );
    openAction_->setToolTip( "Select file and directory" );
    connect( openAction_, SIGNAL( triggered() ), SLOT( open() ) );

    addAction( fileInfoCacheAction_ = new QAction( "File Cache", this ) );
    fileInfoCacheAction_->setToolTip( "Show file cache" );
    connect( fileInfoCacheAction_, SIGNAL( triggered() ), SLOT( _showFileInfoCache() ) );

    addAction( revisionCacheAction_ = new QAction( "Revision Cache", this ) );
    revisionCacheAction_->setToolTip( "Show revision cache" );
    connect( revisionCacheAction_, SIGNAL( triggered() ), SLOT( _showRevisionCache() ) );

    addAction( showCommandDialogAction_ = new QAction( IconEngine::get( ICONS::FIND ), "Show Command List", this ) );
    connect( showCommandDialogAction_, SIGNAL( triggered( void ) ), SLOT( _showCommandDialog( void ) ) );

    addAction( showLogDialogAction_ = new QAction( IconEngine::get( ICONS::FIND ), "Show Log", this ) );
    connect( showLogDialogAction_, SIGNAL( triggered( void ) ), SLOT( _showLogDialog( void ) ) );

}
