
// $Id: RevisionCache.cpp,v 1.12 2012/02/20 20:49:45 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file    RevisionCache.cpp
\brief   store list of file info and cvs information. Used for cache.
\author  Hugo Pereira
\version $Revision: 1.12 $
\date    $Date: 2012/02/20 20:49:45 $
*/

#include "Debug.h"
#include "RevisionCache.h"
#include "XmlDef.h"
#include "XmlString.h"

#include <algorithm>

//____________________________________________________________________________
RevisionCache::RevisionCache( const QDomElement& element ):
    Counter( "RevisionCache" ),
    maxSize_( -1 )
{

    Debug::Throw( "RevisionCache::RevisionCache (dom).\n" );

    // parse children elements
    for(QDomNode child_node = element.firstChild(); !child_node.isNull(); child_node = child_node.nextSibling() )
    {
        QDomElement child_element = child_node.toElement();
        if( child_element.tagName() == XML::REVISION_LIST )
        {

            // prepare revision list
            File file;
            TimeStamp creation;
            Revision::List revisionList;

            // parse attributes
            QDomNamedNodeMap attributes( child_element.attributes() );
            for( unsigned int i=0; i<attributes.length(); i++ )
            {
                QDomAttr attribute( attributes.item( i ).toAttr() );
                if( attribute.isNull() ) continue;
                if( attribute.name() == XML::FILE ) file = XmlString( attribute.value() ).toText();
                else if( attribute.name() == XML::CREATION ) creation = TimeStamp( attribute.value().toInt() );
                else Debug::Throw(0) << "unrecognized attachment attribute: \"" << attribute.name() << "\"\n";
            }

            // parse children
            for(QDomNode list_child_node = child_element.firstChild(); !list_child_node.isNull(); list_child_node = list_child_node.nextSibling() )
            {
                QDomElement list_child_element = list_child_node.toElement();
                if( list_child_element.tagName() == XML::REVISION ) revisionList.push_back( Revision( list_child_element ) );
                else Debug::Throw(0) << "unrecognized child " << child_element.tagName() << ".\n";
            }

            // assign timestamp to revision list
            revisionList.setCreation( creation );

            if( !file.isEmpty() ) insert( file, revisionList );

        }
    }

    return;
}

//____________________________________________________________________________
QDomElement RevisionCache::domElement( QDomDocument& parent ) const
{

    Debug::Throw( "RevisionCache::domElement (dom).\n" );
    QDomElement out( parent.createElement( XML::REVISION_CACHE ) );
    for( const_iterator iter = begin(); iter != end(); ++iter )
    {
        QDomElement list_element( out.appendChild( parent.createElement( XML::REVISION_LIST ) ).toElement() );
        list_element.setAttribute( XML::FILE, XmlString( iter.key() ).toXml() );
        list_element.setAttribute( XML::CREATION, QString().setNum( iter.value().created() ) );
        for( Revision::List::const_iterator revIter = iter.value().begin(); revIter != iter.value().end(); ++revIter )
        { list_element.appendChild( revIter->domElement( parent ) ); }
    }

    return out;

}

//____________________________________________________________________________
RevisionCache& RevisionCache::truncate( void )
{
    Debug::Throw( "RevisionCache::truncate.\n" );

    // check max size
    if( maxSize_ < 0 ) return *this;
    if( maxSize_ == 0 ) { clear(); return *this; }

    // get total size
    int total_size( 0 );
    for( iterator iter = begin(); iter != end(); ++iter )
    { total_size += int( iter.value().size() ); }

    while( total_size > maxSize_ )
    {
        iterator min_iter( std::min_element( begin(), end(), FirstCreatedFTor() ) );
        total_size -= int( min_iter.value().size() );
        erase( min_iter );
    }

    return *this;
}
