#ifndef RevisionFrame_h
#define RevisionFrame_h

// $Id: RevisionFrame.h,v 1.24 2012/09/05 18:48:08 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "CvsInterface.h"
#include "Debug.h"
#include "File.h"
#include "Revision.h"
#include "RevisionCache.h"
#include "RevisionModel.h"

#include <QtCore/QBasicTimer>
#include <QtGui/QWidget>

class AnimatedTreeView;

//! cvs log version list
class RevisionFrame: public QWidget, public Counter
{

    Q_OBJECT

    public:

    //! constructor
    RevisionFrame( QWidget *parent );

    //! destructor
    virtual ~RevisionFrame( void )
    {}

    //! default size
    void setDefaultHeight( const int& value )
    { defaultHeight_ = value; }

    //! default width
    const int& defaultHeight( void ) const
    { return defaultHeight_; }

    //! size
    QSize sizeHint( void ) const
    { return (defaultHeight_ ) >= 0 ? QSize( 0, defaultHeight_ ):QWidget::sizeHint(); }

    //! clear
    void clear( void );

    //! get file
    const File& file( void )
    { return file_; }

    //! cvs interface
    CvsInterface& interface( void )
    { return interface_; }

    //! revision list
    AnimatedTreeView& list( void )
    {
        assert( list_ );
        return *list_;
    }

    //! cache
    RevisionCache& cache( void )
    {

        if( !localCache_.empty() )
        {
            std::sort( localCache_.begin(), localCache_.end() );
            localCache_.erase( std::unique( localCache_.begin(), localCache_.end() ), localCache_.end() );
            cache_[file()] = localCache_;
        }
        return cache_;
    }

    //! cache
    void setCache( const RevisionCache& cache )
    { cache_ = cache; }

    //!@name  actions
    //@{

    //! edit local
    QAction& editLocalAction( void ) const
    { return *editLocalAction_; }

    //! edit revision
    QAction& editRevisionAction( void ) const
    { return *editRevisionAction_; }

    //! diff local and revision
    QAction& diffAction( void ) const
    { return *diffAction_; }

    //@}

    signals:

    //! selection changed
    void selectionChanged( void );

    //! current revision changed
    void currentRevisionChanged( Revision );

    public slots:

    //! set file
    void setFile( const File& file );

    //! cvs command completed
    void commandCompleted( CvsCommand );

    //! clear caches
    void clearCache( void )
    {
        cache_.clear();
        localCache_.clear();
    }

    protected:

    //! timer event
    virtual void timerEvent( QTimerEvent* );

    //! reload revisions using CVS
    void _reload( void );

    protected slots:

    //! update actions based on selection and contents
    void _updateActions( void );

    //! edit local file
    void _editLocal( void );

    //! edit revision
    void _editRevision( void );

    //! diff revision to local
    void _diff( void );

    //! current revision changed
    void _currentRevisionChanged( const QModelIndex& index )
    { if( index.isValid() ) emit currentRevisionChanged( model_.get( index ) ); }

    //!@name slots triggered by CvsInterface
    //@{

    //! display revisions in list
    void _showRevisions( File, Revision::List );

    //! edit revision
    void _editRevision( File source, File destination );

    //! diff revision to local
    void _diff( File source, File destination );

    //@}

    //! store selected jobs in model
    void _storeSelection( void );

    //! restore selected jobs from model
    void _restoreSelection( void );

    private:

    //! install actions
    void _installActions( void );

    //! display revisions in list
    void _showRevisionsFromLocalCache( void );

    //! default width;
    int defaultHeight_;

    //! revision list
    AnimatedTreeView* list_;

    //! model
    RevisionModel model_;

    //! associated file
    File file_;

    //! timer
    /*!
    used to retrieve log from CVS interface,
    with some delay to avoid running too many
    CVS commands when parsing the tree */
    QBasicTimer timer_;

    //! revision cache
    RevisionCache cache_;

    //! local revision cache
    Revision::List localCache_;

    //! cvs interface
    CvsInterface interface_;

    //!@name actions
    //@{

    //! edit local file action
    QAction* editLocalAction_;

    //! edit revision
    QAction* editRevisionAction_;

    //! diff
    QAction* diffAction_;

    //@}

};

#endif
